package com.ruoyi.sensor.service.impl;

import com.ruoyi.board.domain.AlertPlan;
import com.ruoyi.board.domain.BoardInfo;
import com.ruoyi.board.domain.PresetContent;
import com.ruoyi.board.domain.ReleaseRecord;
import com.ruoyi.board.domain.enums.AlertPlanType;
import com.ruoyi.board.service.IAlertPlanService;
import com.ruoyi.board.service.IBoardInfoService;
import com.ruoyi.board.service.IPresetContentService;
import com.ruoyi.board.service.IReleaseRecordService;
import com.ruoyi.protocol.service.impl.ContentPublishingService;
import com.ruoyi.sensor.common.PerceptionHandlerMap;
import com.ruoyi.sensor.domain.PerceptionParams;
import com.ruoyi.sensor.domain.PerceptionWarningStatusInfo;
import com.ruoyi.sensor.domain.PresetContentStyle;
import com.ruoyi.sensor.domain.ReleaseLogReport;
import com.ruoyi.sensor.domain.dto.PerceptionHandlerDTO;
import com.ruoyi.sensor.enums.PrintColor;
import com.ruoyi.sensor.merchants.MerchantsHttp;
import com.ruoyi.sensor.service.ISensorDataService;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class SensorDataServiceImpl implements ISensorDataService {

    private static final Logger log = LoggerFactory.getLogger(SensorDataServiceImpl.class);
    private static final Integer ALL_TIMES_THRESHOLD = 5;
    private static final Integer MIN_TIMES_THRESHOLD = 4;
    @Autowired
    private IBoardInfoService boardInfoService;
    @Autowired
    private IAlertPlanService alertPlanService;
    @Autowired
    private IPresetContentService presetContentService;
    @Autowired
    private IReleaseRecordService releaseRecordService;
    @Autowired
    private ContentPublishingService contentPublishingService;

    @Override
    @Async
    public void perceptionParamsHandler(PerceptionParams params) {
        int currentContentPlanType = AlertPlanType.getPlanTypeByName(params.getPerceptionType());
        // 找到符合计划的预警
        AlertPlan currentAlertPlan = alertPlanService.getOneByTypeAndValue(currentContentPlanType, params.getPerceptionValue());
        if (null == currentAlertPlan) {
            log.error("找不到预警计划");
            return;
        }
        // 情报板设备信息
        BoardInfo boardInfo = boardInfoService.getOneByIP(params.getBoardIp());
        if (null == boardInfo) {
            log.error("找不到情报板信息");
            return;
        }
        // 预置发布的信息
        PresetContent currentContent = presetContentService.getOneByContentAndBoardSize(currentAlertPlan.getDisplayContent(), boardInfo.getBoardSize(), currentContentPlanType);
        if (null == currentContent) {
            log.error("找不到预置发布的信息");
            return;
        }
        // 预警信息发布流程
        List<PresetContent> informationToBeReleasedList = new ArrayList<>();
        String presetStatus = "预警开始";
        // 查询这个情报板 2 分钟内所有发布内容
        List<ReleaseRecord> releaseRecordList = releaseRecordService.listLatest2minRecordByBoardId(boardInfo.getId());
        if (releaseRecordList.isEmpty()) {
            // 如果查到为空，说明 2 分钟内没有发布内容，这里是第一次发布，直接走发布逻辑
            informationToBeReleasedList.add(currentContent);
        } else {
            // 如果不为空，说明 2 分钟内是发布了内容的，查询所有的发布内容
            List<Integer> presetContentIdList = releaseRecordList.stream().map(ReleaseRecord::getPresetContentId).collect(Collectors.toList());
            List<PresetContent> existPresetContentList = presetContentService.listByIds(presetContentIdList);
            // 先把已经存在的都加入进去
            informationToBeReleasedList.addAll(existPresetContentList);
            // 判断所有发布内容中是否存在当前要发布的类型
            PresetContent isExistsPresetContent = existPresetContentList.stream().filter(i -> i.getInfoType() == currentContentPlanType).findFirst().orElse(null);
            if (isExistsPresetContent == null) {
                // 说明这是一个新发布的预警类型
                informationToBeReleasedList.add(currentContent);
            } else {
                // 说明这是一个已有的类型，需要获取预警等级，然后进行判断
                // 判断已有类型是否和现在的一样然后再替换掉
                // 现在需要知道这个同类型的属于是哪个级别的
                int foundPlanId = releaseRecordList.stream()
                        .filter(i -> Objects.equals(i.getPresetContentId(), isExistsPresetContent.getId()))
                        .mapToInt(ReleaseRecord::getPlanId)
                        .findFirst().orElse(-1);
                AlertPlan byId = alertPlanService.getById(foundPlanId);
                int compare = NumberUtils.compare(byId.getLevel(),
                        currentAlertPlan.getLevel());
                switch (compare) {
                    case 0:
                        // 两个一样等级
                        // 这时候就不需要替换，也不需要重新发布
                        presetStatus = "预警持续";
                        break;
                    case 1:
                        // getPresetContentLevel > alertPlan.getLevel()
                        // 预警减弱
                        // 此时需要将同类型的信息给替换掉，并重新发布
                        presetStatus = "预警减弱";
                        break;
                    case -1:
                        // getPresetContentLevel < alertPlan.getLevel()
                        // 预警加强
                        // 此时需要将同类型的信息给替换掉，并重新发布
                        presetStatus = "预警加强";
                        break;
                }
                // 替换同类型的预警信息
                for (int i = 0; i < existPresetContentList.size(); i++) {
                    if (existPresetContentList.get(i).getInfoType() == currentContentPlanType) {
                        existPresetContentList.set(i, currentContent);
                    }
                }
            }
        }
        // todo 发布
        contentPublishingService.contentPublish(boardInfo, informationToBeReleasedList);
        // 经过主动发布之后就变成预置信息
        // 发布之后记录发布内容 ReleaseRecord
        this.saveRecord(boardInfo.getId(), currentContent.getId(), currentAlertPlan.getId());
        // 记录完了之后就需要向对方发送状态日志记录
        this.sendStatus(presetStatus, boardInfo.getBoardIp(), params.getPerceptionType(), currentContent.getContent(), params.getPerceptionValue(), currentAlertPlan.getLevel());
        // 发送情报板发布信息日志记录
        this.sendPublicLog(boardInfo.getBoardIp(), params.getPerceptionType(), currentAlertPlan.getLevel(), currentContent.getContent(), informationToBeReleasedList);
    }

    @Override
    public void putPerceptionParamsToMap(PerceptionParams perceptionParams) {
        String key = perceptionParams.getPerceptionType() + perceptionParams.getBoardIp();
        PerceptionHandlerDTO orDefault = PerceptionHandlerMap.perceptionHandlerMap.getOrDefault(key, new PerceptionHandlerDTO());
        orDefault.setPerceptionParams(perceptionParams);
        orDefault.setAllReceiveTimes(orDefault.getAllReceiveTimes() + 1);
        orDefault.setCurrentMinReceiveTimes(orDefault.getCurrentMinReceiveTimes() + 1);
        orDefault.setUpdateTime(new Date());
        PerceptionHandlerMap.perceptionHandlerMap.put(key, orDefault);
    }



    @Override
    public void handlerPerceptionMap(){
        List<PerceptionParams> publishList = new ArrayList<>();
        List<PerceptionParams> cancelList = new ArrayList<>();
        List<String> delList = new ArrayList<>();
        long now = new Date().getTime();
        log.info("{}", PerceptionHandlerMap.perceptionHandlerMap);
        PerceptionHandlerMap.perceptionHandlerMap.forEach((key, value) -> {
            long lastUpdateTime = value.getUpdateTime().getTime();
            // 最后的接收时间大于 1 分钟
            if (now - lastUpdateTime > 60 * 1000) {
                delList.add(key);
                if (value.getAllReceiveTimes() > ALL_TIMES_THRESHOLD ) {
                    cancelList.add(value.getPerceptionParams());
                }
            } else {
                // 接收时间 < 1 分钟 && 接收次数 > 5
                // 要不要 + 个属性，当前分钟内接收的次数，以及接收的总次数
                if (value.getAllReceiveTimes() > ALL_TIMES_THRESHOLD && value.getCurrentMinReceiveTimes() > MIN_TIMES_THRESHOLD) {
                    publishList.add(value.getPerceptionParams());
                }
            }
            value.setCurrentMinReceiveTimes(0);
        });
        // 需要删除的内容，有的是没有发布的浮动信息，有的是已经显示过了，需要取消显示的
        cancelList.forEach(this::cancelPublication);
        log.info("cancelPublication:{}", cancelList);
        delList.forEach(PerceptionHandlerMap.perceptionHandlerMap::remove);
        log.info("delList:{}", delList);
        publishList.forEach(this::perceptionParamsHandler);
        log.info("publishList:{}", publishList);
    }

    private void sendPublicLog(String ip, String type, Integer level, String content, List<PresetContent> informationToBeReleasedList) {
        ReleaseLogReport releaseLogReport = new ReleaseLogReport();
        releaseLogReport.setBoardIp(ip);
        releaseLogReport.setPerceptionType(type);
        releaseLogReport.setPerceptionLevel(level);
        releaseLogReport.setReleaseContent(content);
        releaseLogReport.setReleaseTime(LocalDateTime.now());
        releaseLogReport.setReleaseStatus("发布成功");
        List<PresetContentStyle> presetContentStyles = new ArrayList<>();
        informationToBeReleasedList.forEach(i -> {
            String[] split = StringUtils.split(i.getBoardSize(), "*");
            PresetContentStyle p = new PresetContentStyle();
            p.setFont(i.getFontStyle());
            p.setFontSize(i.getFontSize());
            p.setFontColor(PrintColor.getBitColor(i.getFontColor()));
            p.setContent(i.getContent());
            p.setStartX(i.getFontPositionX());
            p.setStartY(i.getFontPositionY());
            p.setWidth(NumberUtils.toInt(split[1]));
            p.setHeight(NumberUtils.toInt(split[0]));
            p.setStay(i.getPlayTime());
            presetContentStyles.add(p);
        });
        releaseLogReport.setLines(presetContentStyles);
        MerchantsHttp.sendPublicationLog(releaseLogReport);
    }

    private void sendStatus(String status, String ip, String type, String content, Double value, Integer level) {
        PerceptionWarningStatusInfo statusInfo = new PerceptionWarningStatusInfo();
        statusInfo.setPerceptionStatus(status);
        statusInfo.setBoardIp(ip);
        statusInfo.setPerceptionType(type);
        statusInfo.setPerceptionValue(value);
        statusInfo.setPerceptionMsg(content);
        statusInfo.setPerceptionLevel(level);
        statusInfo.setPerceptionTime(LocalDateTime.now());
//        // 开始发送状态记录
        MerchantsHttp.sendWarningStatus(statusInfo);
    }

    @Async
    public void cancelPublication(PerceptionParams params) {
        PerceptionWarningStatusInfo statusInfo = new PerceptionWarningStatusInfo();
        statusInfo.setPerceptionStatus("预警结束");
        statusInfo.setBoardIp(params.getBoardIp());
        statusInfo.setPerceptionType(params.getPerceptionType());
        statusInfo.setPerceptionValue(params.getPerceptionValue());
        statusInfo.setPerceptionMsg("");
        statusInfo.setPerceptionLevel(1);
        statusInfo.setPerceptionTime(LocalDateTime.now());
//        // 开始发送状态记录
        MerchantsHttp.sendWarningStatus(statusInfo);
    }

    private void saveRecord(Integer boardId, Integer presetContentId, Integer planId) {
        ReleaseRecord releaseRecord = new ReleaseRecord();
        releaseRecord.setBoardId(boardId);
        releaseRecord.setPresetContentId(presetContentId);
        releaseRecord.setPlanId(planId);
        releaseRecord.setCreateTime(new Date());
        releaseRecord.setCreatedBy(1);
        releaseRecordService.save(releaseRecord);
    }
}
